from json import JSONDecodeError
import logging
import time
from typing import Dict

import jwt
import requests
import attr
from typing_extensions import Literal

from pyzoom import err


@attr.s
class APIClientBase:
    api_key: str = attr.ib(repr=False)
    api_secret: str = attr.ib(repr=False)

    name = "zoom_api_client"
    user_id: str = "me"
    base_url: str = "https://api.zoom.us/v2"

    def bearer_token(self) -> str:
        return self.generate_jwt(self.api_key, self.api_secret)

    def make_request(
        self,
        endpoint: str,
        method: Literal["GET", "POST", "PATCH", "DELETE", "PUT"],
        query: Dict = None,
        body: Dict = None,
        raise_on_error=True,
    ) -> requests.Response:
        allowed_methods = "GET POST PATCH DELETE PUT".split()
        if method not in allowed_methods:
            raise ValueError(
                f'Invalid method: {method}. Must be one of {", ".join(allowed_methods)}'
            )
        headers = {"Authorization": f"Bearer {self.bearer_token()}"}
        url = self.base_url + endpoint
        logging.debug(f"Making {method} request to {endpoint}")
        session = requests.Session()

        r = session.request(method, url, headers=headers, params=query, json=body)

        if 200 <= r.status_code < 300:
            return r

        try:
            body = r.json()
            try:
                message = body["_error"]["message"]
            except KeyError:
                message = body
        except JSONDecodeError:
            body = r.text
            message = body

        logging.error(f"Unsuccessful request to {r.url}: [{r.status_code}] {message}")

        logging.debug(f"Full response: {body}")
        logging.debug(f"Headers: {r.headers}")
        logging.debug(f"Params: {query}")
        logging.debug(f"Body: {body}")
        if not raise_on_error:
            logging.warning(
                f"raise_on_error is False, ignoring API error and returning response"
            )
            return r

        if r.status_code in err.HTTP_ERRORS_MAP:
            raise err.HTTP_ERRORS_MAP[r.status_code](message)
        else:
            raise err.APIError(message)

    def get(
        self, endpoint: str, query: Dict = None, raise_on_error: bool = True
    ) -> requests.Response:
        return self.make_request(
            endpoint, method="GET", query=query, raise_on_error=raise_on_error
        )

    def get_all_pages(
        self, endpoint: str, query: Dict = None, raise_on_error: bool = True
    ) -> Dict:
        res = self.get(endpoint, query=query, raise_on_error=raise_on_error).json()
        next_page_token = res.get("next_page_token")
        while next_page_token:
            next_page_res = self.get(
                endpoint,
                query={"next_page_token": next_page_token},
                raise_on_error=raise_on_error,
            ).json()
            next_page_token = next_page_res.get("next_page_token")
            for k, v in next_page_res.items():
                if isinstance(v, list):
                    res[k].extend(v)
        res["next_page_token"] = next_page_token
        return res

    def post(
        self,
        endpoint: str,
        query: Dict = None,
        body: Dict = None,
        raise_on_error: bool = True,
    ) -> requests.Response:
        return self.make_request(
            endpoint,
            method="POST",
            query=query,
            body=body,
            raise_on_error=raise_on_error,
        )

    def patch(
        self,
        endpoint: str,
        query: Dict = None,
        body: Dict = None,
        raise_on_error: bool = True,
    ) -> requests.Response:
        return self.make_request(
            endpoint,
            method="PATCH",
            query=query,
            body=body,
            raise_on_error=raise_on_error,
        )

    def put(
        self,
        endpoint: str,
        query: Dict = None,
        body: Dict = None,
        raise_on_error: bool = True,
    ) -> requests.Response:
        return self.make_request(
            endpoint,
            method="PUT",
            query=query,
            body=body,
            raise_on_error=raise_on_error,
        )

    def delete(
        self,
        endpoint: str,
        query: Dict = None,
        body: Dict = None,
        raise_on_error: bool = True,
    ) -> requests.Response:
        return self.make_request(
            endpoint,
            method="DELETE",
            query=query,
            body=body,
            raise_on_error=raise_on_error,
        )

    @staticmethod
    def generate_jwt(key, secret):
        header = {"alg": "HS256", "typ": "JWT"}

        payload = {"iss": key, "exp": int(time.time() + 3600)}

        token = jwt.encode(payload, secret, algorithm="HS256", headers=header)

        # Compatibility between different versions of pyjwt (2.1.0 returns str).
        return token if isinstance(token, str) else token.decode("utf-8")



from __future__ import annotations

import os
from typing import List

import attr
import shortuuid
from typing_extensions import Literal

from pyzoom._base import APIClientBase
from pyzoom import schemas


@attr.s
class MeetingsComponent:
    _client: APIClientBase = attr.ib(repr=False)
    timezone: str = attr.ib(default="UTC")

    def list_meetings(self) -> schemas.ZoomMeetingShortList:
        endpoint = "/users/me/meetings"
        return schemas.ZoomMeetingShortList(**self._client.get(endpoint).json())

    def get_meeting(self, meeting_id: int) -> schemas.ZoomMeeting:
        endpoint = f"/meetings/{meeting_id}"
        return schemas.ZoomMeeting(**self._client.get(endpoint).json())
    """
    LineNumber:49 can be without Conversion to dict. Which may give user 
    an advantage to pass dictionary while creating the meeting in parameter 'settings'.
    """
    def create_meeting(
        self,
        topic: str,
        *,
        start_time: str,
        duration_min: int,
        timezone: str = None,
        type_: int = 2,
        password: str = None,
        settings: schemas.ZoomMeetingSettings = None,
    ) -> schemas.ZoomMeeting:
        endpoint = f"/users/me/meetings"
        body = {
            "topic": topic,
            "type": type_,
            "start_time": start_time,
            "duration": duration_min,
            "timezone": timezone or self.timezone,
            "password": password or shortuuid.random(6),
            "settings": settings.dict()
            if settings
            else schemas.ZoomMeetingSettings.default_settings().dict(),
        }
        response = self._client.post(endpoint, body=body)
        return schemas.ZoomMeeting(**response.json())

    """
    UpdateMeeting Should be passed with same parameters as create_meeting but
    meeting_id is an additional requirement when we wish to modify the meeting.
    In the return from we get a StatusCode and empty dictionary.
    """

    def update_meeting(
        self,
        topic: str,
        *,
        start_time: str,
        duration_min: int,
        timezone: str = None,
        type_: int = 2,
        password: str = None,
        settings: schemas.ZoomMeetingSettings = None,
        meeting_id: int,
    ) -> schemas.ZoomMeeting:
        endpoint = f"/meetings/{meeting_id}"
        body = {
            "topic": topic,
            "type": type_,
            "start_time": start_time,
            "duration": duration_min,
            "timezone": timezone or self.timezone,
            "password": password or shortuuid.random(6),
            "settings": settings.dict() 
            if settings
            else schemas.ZoomMeetingSettings.default_settings().dict(),
        }
        response = self._client.patch(endpoint, body=body)
        return {}

    def delete_meeting(self, meeting_id: int) -> bool:
        endpoint = f"/meetings/{meeting_id}"
        r = self._client.delete(endpoint)
        return r.status_code == 204

    def list_meeting_registrants(
        self, meeting_id: int
    ) -> schemas.MeetingRegistrantsList:
        endpoint = f"/meetings/{meeting_id}/registrants"
        return schemas.MeetingRegistrantsList(**self._client.get_all_pages(endpoint))

    def update_meeting_registrant_status(
        self,
        meeting_id: int,
        *,
        action: Literal["approve", "cancel", "deny"],
        registrants: List[schemas.MeetingRegistrantShort],
    ):
        endpoint = f"/meetings/{meeting_id}/registrants/status"
        body = {
            "action": action,
            "registrants": [registrant.dict() for registrant in registrants],
        }
        return self._client.put(endpoint, body=body)

    def add_meeting_registrant(
        self, meeting_id: int, *, first_name: str, last_name: str, email: str, **kwargs
    ) -> schemas.RegistrantConfirmation:
        endpoint = f"/meetings/{meeting_id}/registrants"
        registrant = schemas.MeetingRegistrant(
            first_name=first_name, last_name=last_name, email=email, **kwargs
        )
        return schemas.RegistrantConfirmation(
            **self._client.post(endpoint, body=registrant.dict()).json()
        )

    def add_and_confirm_registrant(
        self, meeting_id: int, *, first_name: str, last_name: str, email: str, **kwargs
    ) -> schemas.RegistrantConfirmation:
        confirmation = self.add_meeting_registrant(
            meeting_id,
            first_name=first_name,
            last_name=last_name,
            email=email,
            **kwargs,
        )
        self.update_meeting_registrant_status(
            meeting_id,
            action="approve",
            registrants=[
                schemas.MeetingRegistrantShort(
                    id=confirmation.registrant_id, email=email
                )
            ],
        )
        return confirmation

    def cancel_registration(self, meeting_id: int, *, registrant_id: str, email: str):
        self.update_meeting_registrant_status(
            meeting_id,
            action="cancel",
            registrants=[schemas.MeetingRegistrantShort(id=registrant_id, email=email)],
        )

    def approve_registration(self, meeting_id: int, *, registrant_id: str, email: str):
        self.update_meeting_registrant_status(
            meeting_id,
            action="approve",
            registrants=[schemas.MeetingRegistrantShort(id=registrant_id, email=email)],
        )

    def past_meeting_participants(
        self, meeting_id: int
    ) -> schemas.MeetingParticipantList:
        endpoint = f"/past_meetings/{meeting_id}/participants"
        return schemas.MeetingParticipantList(**self._client.get_all_pages(endpoint))

@attr.s
class UsersComponent:
    _client: APIClientBase = attr.ib(repr=False)

    def get_users(
        self, status: str
    ) -> schemas.ZoomUserList:
        endpoint = f"/users"
        query={
                "status": status,
        }
        return schemas.ZoomUserList(**self._client.get_all_pages(endpoint, query))

    def delete_user(self, user_id: int, ) -> bool:
        endpoint = f"/users/{user_id}"
        query={
            "action": "delete"
        }
        r = self._client.delete(endpoint, query)
        return r.status_code == 204

@attr.s(auto_attribs=True)
class ZoomClient:
    api_key: str = attr.ib(repr=False)
    api_secret: str = attr.ib(repr=False)

    raw: APIClientBase = attr.ib(init=False, repr=False)
    meetings: MeetingsComponent = attr.ib(init=False, repr=False)
    users: UsersComponent = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.raw: APIClientBase = APIClientBase(
            api_key=self.api_key, api_secret=self.api_secret
        )
        self.meetings: MeetingsComponent = MeetingsComponent(self.raw)
        self.users: UsersComponent = UsersComponent(self.raw)

    @classmethod
    def from_environment(cls) -> ZoomClient:
        env = os.environ
        return cls(api_key=env["ZOOM_API_KEY"], api_secret=env["ZOOM_API_SECRET"])

    def set_timezone(self, timezone: str):
        self.meetings.timezone = timezone



from __future__ import annotations

from typing import Optional, List

from pydantic import BaseModel
from typing_extensions import Literal


class MyZoomBase(BaseModel):
    class Config:
        orm_mode = True

    def pprint(self):
        from pprint import pprint

        pprint(self.dict())


class ZoomMeetingSettings(MyZoomBase):
    host_video: bool
    participant_video: bool
    cn_meeting: bool
    in_meeting: bool
    join_before_host: bool
    mute_upon_entry: bool
    watermark: bool
    use_pmi: bool
    approval_type: Literal[0, 1, 2]
    registration_type: Optional[Literal[1, 2, 3]]
    audio: Literal["voip", "telephony", "both"]
    auto_recording: Literal["local", "cloud", "none"]
    enforce_login: bool
    enforce_login_domains: Optional[str]
    alternative_hosts: Optional[str]
    close_registration: Optional[bool]
    waiting_room: bool
    global_dial_in_countries: Optional[List[str]]
    contact_name: Optional[str]
    contact_email: Optional[str]
    registrants_email_notification: bool
    meeting_authentication: bool
    authentication_option: Optional[str]
    authentication_domains: Optional[str]

    @classmethod
    def default_settings(cls) -> ZoomMeetingSettings:
        return ZoomMeetingSettings(
            host_video=True,
            participant_video=True,
            join_before_host=True,
            mute_upon_entry=True,
            approval_type=0,
            cn_meeting=False,
            in_meeting=False,
            watermark=False,
            use_pmi=False,
            registration_type=1,
            audio="voip",
            auto_recording="none",
            enforce_login=True,
            waiting_room=False,
            registrants_email_notification=False,
            meeting_authentication=True,
            contact_name="UFY",
            contact_email="ufyit@support.com",
        )


class ZoomMeetingShort(MyZoomBase):
    uuid: str
    id: int
    host_id: str
    topic: str
    type: int
    start_time: str
    duration: int
    timezone: str
    created_at: str
    join_url: str


class ZoomMeeting(ZoomMeetingShort):
    status: str
    agenda: Optional[str]
    start_url: str
    registration_url: Optional[str]
    password: str
    h323_password: str
    pstn_password: str
    encrypted_password: str
    settings: ZoomMeetingSettings


class ZoomMeetingShortList(MyZoomBase):
    page_count: int
    page_number: int
    page_size: int
    total_records: int
    meetings: List[ZoomMeetingShort]

    def filter_by_topic(self, text: str) -> List[ZoomMeetingShort]:
        return [m for m in self.meetings if text.lower() in m.topic.lower()]

    def filter_by_id(self, meeting_id: int) -> List[ZoomMeetingShort]:
        return [m for m in self.meetings if m.id == meeting_id]


class MeetingRegistrantShort(MyZoomBase):
    id: Optional[str]
    email: Optional[str]


class MeetingRegistrant(MeetingRegistrantShort):
    email: str
    first_name: str
    last_name: str
    address: Optional[str]
    city: Optional[str]
    country: Optional[str]
    zip: Optional[str]
    state: Optional[str]
    phone: Optional[str]
    industry: Optional[str]
    org: Optional[str]
    job_title: Optional[str]
    comment: Optional[str]


class MeetingRegistrantsList(MyZoomBase):
    page_count: int
    page_number: int
    page_size: int
    total_records: int
    registrants: List[MeetingRegistrant]


class RegistrantConfirmation(MyZoomBase):
    registrant_id: str
    id: int
    topic: str
    start_time: str
    join_url: str


class MeetingParticipant(MyZoomBase):
    id: str
    name: str
    user_email: str


class MeetingParticipantList(MyZoomBase):
    page_count: int
    page_size: int
    total_records: int
    participants: Optional[List[MeetingParticipant]]

    def __len__(self):
        return len(self.participants)

    def __iter__(self):
        return iter(self.participants)

    def __getitem__(self, index):
        return self.participants[index]

    def find_by_id(self, id_: str) -> List[MeetingParticipant]:
        return [i for i in self.participants if i.id == id_]

    def find_by_email(self, email: str) -> List[MeetingParticipant]:
        return [i for i in self.participants if i.user_email == email]

    def find_by_name(self, name):
        return [i for i in self.participants if i.name == name]

class ZoomUser(MyZoomBase):
    id: str
    first_name: str
    last_name: str
    email: str
    type: int
    pmi: int
    timezone: Optional[str]
    verified: int
    dept: Optional[str]
    created_at: str
    pic_url: Optional[str]
    group_ids: Optional[List[str]]
    language: Optional[str]
    phone_number: Optional[str]
    status: str
    role_id: str

class ZoomUserList(MyZoomBase):
    page_count: int
    page_number: int
    page_size: int
    total_records: int
    users: List[ZoomUser]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyzoom']

package_data = \
{'': ['*']}

install_requires = \
['attrs',
 'cachetools>=4.1,<6.0',
 'pydantic>=1.5.1,<2.0.0',
 'pyjwt>=2.0,<3.0.0',
 'requests>=2.23.0,<3.0.0',
 'shortuuid>=1.0.1,<2.0.0',
 'typing-extensions>=3.7.4,<5.0.0']

setup_kwargs = {
    'name': 'pyzoom',
    'version': '0.1.15',
    'description': 'Python wrapper for Zoom Video API',
    'long_description': '![Zoom Logo](https://d24cgw3uvb9a9h.cloudfront.net/static/93946/image/new/ZoomLogo.png)\n\n# Python wrapper for Zoom API\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pyzoom)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![PyPI](https://img.shields.io/pypi/v/pyzoom)](https://pypi.org/project/pyzoom/)\n![PyPI - License](https://img.shields.io/pypi/l/pyzoom)\n![PyPI - Downloads](https://img.shields.io/pypi/dw/pyzoom)\n[![](https://img.shields.io/badge/Support-Buy_coffee!-Orange)](https://www.buymeacoffee.com/licht1stein)\n\nThis library is work in progress, and that includes documentation. Not all of the implemented methods are documented here,\nso please explore the `ZoomClient` class.\n\nLinks:\n* [Api Reference](https://marketplace.zoom.us/docs/api-reference)\n* [Using Zoom API](https://marketplace.zoom.us/docs/api-reference/using-zoom-apis)\n\n## Installation\n\nUsing pip:\n\n`pip install -U pyzoom`\n\nUsing [poetry](https://python-poetry.org/):\n\n`poetry add pyzoom`\n\n## Usage\n\n### Basic instantiation:\n\n```python\nfrom pyzoom import ZoomClient\n\nclient = ZoomClient(\'YOUR_ZOOM_API_KEY\', \'YOUR_ZOOM_API_SECRET\')\n```\n\n### Instantiation from environment variables\n\nYou can also create an instance of client when storing your key and secret in environment variables `ZOOM_API_KEY` \nand `ZOOM_API_SECRET`.\n\n```python\nfrom pyzoom import ZoomClient\n\nclient = ZoomClient.from_environment()\n```\n\n\n### Meetings\n\n#### Create meeting, update meeting and add registrant\n```python\nfrom pyzoom import ZoomClient\nfrom datetime import datetime as dt\n\nclient = ZoomClient.from_environment()\n\n# Creating a meeting\nmeeting = client.meetings.create_meeting(\'Auto created 1\', start_time=dt.now().isoformat(), duration_min=60, password=\'not-secure\')\n\n# Update a meeting\nmeeting = client.meetings.update_meeting(\'Auto updated 1\', meeting_id = meeting.id ,start_time=dt.now().isoformat(), duration_min=60,password=\'not-secure\')\n\n# Adding registrants\nclient.meetings.add_meeting_registrant(meeting.id, first_name=\'John\', last_name=\'Doe\', email=\'john.doe@example.com\')\n```\nYou can use `client.meetings.add_and_confirm_registrant` to also confirm auto added\nregistrants to a closed meeting.\n\n### Raw API methods\n\nYou can also use the library for making raw requests to the API:\n\n```python\nfrom pyzoom import ZoomClient\n\nclient = ZoomClient.from_environment()\n\n# Get self\nresponse = client.raw.get(\'/me\')\n\n# Get all pages of meeting participants\nresult_dict = client.raw.get_all_pages(\'/past_meetings/{meetingUUID}/participants\')\n```\n\n### Packaging notice\nThis project uses the excellent [poetry](https://python-poetry.org) for packaging. Please read about it and let\'s all start using\n`pyproject.toml` files as a standard. Read more:\n\n* [PEP 518 -- Specifying Minimum Build System Requirements for Python Projects](https://www.python.org/dev/peps/pep-0518/)\n\n* [What the heck is pyproject.toml?](https://snarky.ca/what-the-heck-is-pyproject-toml/)\n\n* [Clarifying PEP 518 (a.k.a. pyproject.toml)](https://snarky.ca/clarifying-pep-518/)\n\n\n### Support\n\n<a href="https://www.buymeacoffee.com/licht1stein" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/default-orange.png" alt="Buy Me A Coffee" style="height: 30px !important;width: 130px !important;" ></a>\n\n### Disclaimer\nThis library is not related to Zoom Video Communications, Inc. It\'s an open-source project that \naims to simplify working with this suddenly very popular service.\n',
    'author': 'MB',
    'author_email': 'mb@blaster.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/licht1stein/pyzoom',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



class BaseError(Exception):
    pass


class InvalidData(Exception):
    pass


class APIError(BaseError):
    pass


class NotFound(APIError):
    pass


class BadRequest(APIError):
    pass


class NotAuthorized(APIError):
    pass


class NotAllowed(APIError):
    pass


class Conflict(APIError):
    pass


HTTP_ERRORS_MAP = {
    400: BadRequest,
    401: NotAuthorized,
    404: NotFound,
    405: NotAllowed,
    409: Conflict,
}
